@@ -70,6 +70,8 @@ __all__ = [
     "last_event_id",
     "new_scope",
     "push_scope",
+    "remove_attribute",
+    "set_attribute",
     "set_context",
     "set_extra",
     "set_level",
@@ -287,6 +289,28 @@ def push_scope(  # noqa: F811
     return _ScopeManager()
 
 
+@scopemethod
+def set_attribute(attribute: str, value: "Any") -> None:
+    """
+    Set an attribute.
+
+    Any attributes-based telemetry (logs, metrics) captured in this scope will
+    include this attribute.
+    """
+    return get_isolation_scope().set_attribute(attribute, value)
+
+
+@scopemethod
+def remove_attribute(attribute: str) -> None:
+    """
+    Remove an attribute.
+
+    If the attribute doesn't exist, this function will not have any effect and
+    it will also not raise an exception.
+    """
+    return get_isolation_scope().remove_attribute(attribute)
+
+
 @scopemethod
 def set_tag(key: str, value: "Any") -> None:
     return get_isolation_scope().set_tag(key, value)
