@@ -50,7 +50,7 @@ if TYPE_CHECKING:
     from sentry_sdk.tracing import Span
     from sentry_sdk._types import TextPart
 
-    from openai.types.responses import ResponseInputParam
+    from openai.types.responses import ResponseInputParam, SequenceNotStr
     from openai import Omit
 
 try:
@@ -220,20 +220,6 @@ def _calculate_token_usage(
     )
 
 
-def _get_input_messages(
-    kwargs: "dict[str, Any]",
-) -> "Optional[Union[Iterable[Any], list[str]]]":
-    # Input messages (the prompt or data sent to the model)
-    messages = kwargs.get("messages")
-    if messages is None:
-        messages = kwargs.get("input")
-
-    if isinstance(messages, str):
-        messages = [messages]
-
-    return messages
-
-
 def _commmon_set_input_data(
     span: "Span",
     kwargs: "dict[str, Any]",
@@ -371,13 +357,6 @@ def _set_completions_api_input_data(
         _commmon_set_input_data(span, kwargs)
         return
 
-    system_instructions = _get_system_instructions_completions(messages)
-    if len(system_instructions) > 0:
-        span.set_data(
-            SPANDATA.GEN_AI_SYSTEM_INSTRUCTIONS,
-            json.dumps(_transform_system_instructions(system_instructions)),
-        )
-
     if isinstance(messages, str):
         normalized_messages = normalize_message_roles([messages])  # type: ignore
         scope = sentry_sdk.get_current_scope()
@@ -390,13 +369,29 @@ def _set_completions_api_input_data(
         _commmon_set_input_data(span, kwargs)
         return
 
+    # dict special case following https://github.com/openai/openai-python/blob/3e0c05b84a2056870abf3bd6a5e7849020209cc3/src/openai/_utils/_transform.py#L194-L197
+    if not isinstance(messages, Iterable) or isinstance(messages, dict):
+        set_data_normalized(span, SPANDATA.GEN_AI_OPERATION_NAME, "chat")
+        _commmon_set_input_data(span, kwargs)
+        return
+
+    messages = list(messages)
+    kwargs["messages"] = messages
+
+    system_instructions = _get_system_instructions_completions(messages)
+    if len(system_instructions) > 0:
+        span.set_data(
+            SPANDATA.GEN_AI_SYSTEM_INSTRUCTIONS,
+            json.dumps(_transform_system_instructions(system_instructions)),
+        )
+
     non_system_messages = [
         message
         for message in messages
         if not _is_system_instruction_completions(message)
     ]
     if len(non_system_messages) > 0:
-        normalized_messages = normalize_message_roles(non_system_messages)  # type: ignore
+        normalized_messages = normalize_message_roles(non_system_messages)
         scope = sentry_sdk.get_current_scope()
         messages_data = truncate_and_annotate_messages(normalized_messages, span, scope)
         if messages_data is not None:
@@ -413,15 +408,47 @@ def _set_embeddings_input_data(
     kwargs: "dict[str, Any]",
     integration: "OpenAIIntegration",
 ) -> None:
-    messages = _get_input_messages(kwargs)
+    messages: "Union[str, SequenceNotStr[str], Iterable[int], Iterable[Iterable[int]]]" = kwargs.get(
+        "input"
+    )
 
     if (
-        messages is not None
-        and len(messages) > 0  # type: ignore
-        and should_send_default_pii()
-        and integration.include_prompts
+        not should_send_default_pii()
+        or not integration.include_prompts
+        or messages is None
     ):
-        normalized_messages = normalize_message_roles(messages)  # type: ignore
+        set_data_normalized(span, SPANDATA.GEN_AI_OPERATION_NAME, "embeddings")
+        _commmon_set_input_data(span, kwargs)
+
+        return
+
+    if isinstance(messages, str):
+        set_data_normalized(span, SPANDATA.GEN_AI_OPERATION_NAME, "embeddings")
+        _commmon_set_input_data(span, kwargs)
+
+        normalized_messages = normalize_message_roles([messages])  # type: ignore
+        scope = sentry_sdk.get_current_scope()
+        messages_data = truncate_and_annotate_embedding_inputs(
+            normalized_messages, span, scope
+        )
+        if messages_data is not None:
+            set_data_normalized(
+                span, SPANDATA.GEN_AI_EMBEDDINGS_INPUT, messages_data, unpack=False
+            )
+
+        return
+
+    # dict special case following https://github.com/openai/openai-python/blob/3e0c05b84a2056870abf3bd6a5e7849020209cc3/src/openai/_utils/_transform.py#L194-L197
+    if not isinstance(messages, Iterable) or isinstance(messages, dict):
+        set_data_normalized(span, SPANDATA.GEN_AI_OPERATION_NAME, "embeddings")
+        _commmon_set_input_data(span, kwargs)
+        return
+
+    messages = list(messages)
+    kwargs["input"] = messages
+
+    if len(messages) > 0:
+        normalized_messages = normalize_message_roles(messages)
         scope = sentry_sdk.get_current_scope()
         messages_data = truncate_and_annotate_embedding_inputs(
             normalized_messages, span, scope
@@ -435,28 +462,16 @@ def _set_embeddings_input_data(
     _commmon_set_input_data(span, kwargs)
 
 
-def _set_output_data(
+def _set_common_output_data(
     span: "Span",
     response: "Any",
-    kwargs: "dict[str, Any]",
+    input: "Any",
     integration: "OpenAIIntegration",
-    start_time: "Optional[float]" = None,
     finish_span: bool = True,
 ) -> None:
     if hasattr(response, "model"):
         set_data_normalized(span, SPANDATA.GEN_AI_RESPONSE_MODEL, response.model)
 
-    # Input messages (the prompt or data sent to the model)
-    # used for the token usage calculation
-    messages = kwargs.get("messages")
-    if messages is None:
-        messages = kwargs.get("input")
-
-    if messages is not None and isinstance(messages, str):
-        messages = [messages]
-
-    ttft: "Optional[float]" = None
-
     if hasattr(response, "choices"):
         if should_send_default_pii() and integration.include_prompts:
             response_text = [
@@ -467,7 +482,7 @@ def _set_output_data(
             if len(response_text) > 0:
                 set_data_normalized(span, SPANDATA.GEN_AI_RESPONSE_TEXT, response_text)
 
-        _calculate_token_usage(messages, response, span, None, integration.count_tokens)
+        _calculate_token_usage(input, response, span, None, integration.count_tokens)
 
         if finish_span:
             span.__exit__(None, None, None)
@@ -503,149 +518,12 @@ def _set_output_data(
                     span, SPANDATA.GEN_AI_RESPONSE_TEXT, output_messages["response"]
                 )
 
-        _calculate_token_usage(messages, response, span, None, integration.count_tokens)
+        _calculate_token_usage(input, response, span, None, integration.count_tokens)
 
         if finish_span:
             span.__exit__(None, None, None)
-
-    elif hasattr(response, "_iterator"):
-        data_buf: "list[list[str]]" = []  # one for each choice
-
-        old_iterator = response._iterator
-
-        def new_iterator() -> "Iterator[ChatCompletionChunk]":
-            nonlocal ttft
-            count_tokens_manually = True
-            for x in old_iterator:
-                with capture_internal_exceptions():
-                    # OpenAI chat completion API
-                    if hasattr(x, "choices"):
-                        choice_index = 0
-                        for choice in x.choices:
-                            if hasattr(choice, "delta") and hasattr(
-                                choice.delta, "content"
-                            ):
-                                if start_time is not None and ttft is None:
-                                    ttft = time.perf_counter() - start_time
-                                content = choice.delta.content
-                                if len(data_buf) <= choice_index:
-                                    data_buf.append([])
-                                data_buf[choice_index].append(content or "")
-                            choice_index += 1
-
-                    # OpenAI responses API
-                    elif hasattr(x, "delta"):
-                        if start_time is not None and ttft is None:
-                            ttft = time.perf_counter() - start_time
-                        if len(data_buf) == 0:
-                            data_buf.append([])
-                        data_buf[0].append(x.delta or "")
-
-                    # OpenAI responses API end of streaming response
-                    if RESPONSES_API_ENABLED and isinstance(x, ResponseCompletedEvent):
-                        _calculate_token_usage(
-                            messages,
-                            x.response,
-                            span,
-                            None,
-                            integration.count_tokens,
-                        )
-                        count_tokens_manually = False
-
-                yield x
-
-            with capture_internal_exceptions():
-                if ttft is not None:
-                    set_data_normalized(
-                        span, SPANDATA.GEN_AI_RESPONSE_TIME_TO_FIRST_TOKEN, ttft
-                    )
-                if len(data_buf) > 0:
-                    all_responses = ["".join(chunk) for chunk in data_buf]
-                    if should_send_default_pii() and integration.include_prompts:
-                        set_data_normalized(
-                            span, SPANDATA.GEN_AI_RESPONSE_TEXT, all_responses
-                        )
-                    if count_tokens_manually:
-                        _calculate_token_usage(
-                            messages,
-                            response,
-                            span,
-                            all_responses,
-                            integration.count_tokens,
-                        )
-
-            if finish_span:
-                span.__exit__(None, None, None)
-
-        async def new_iterator_async() -> "AsyncIterator[ChatCompletionChunk]":
-            nonlocal ttft
-            count_tokens_manually = True
-            async for x in old_iterator:
-                with capture_internal_exceptions():
-                    # OpenAI chat completion API
-                    if hasattr(x, "choices"):
-                        choice_index = 0
-                        for choice in x.choices:
-                            if hasattr(choice, "delta") and hasattr(
-                                choice.delta, "content"
-                            ):
-                                if start_time is not None and ttft is None:
-                                    ttft = time.perf_counter() - start_time
-                                content = choice.delta.content
-                                if len(data_buf) <= choice_index:
-                                    data_buf.append([])
-                                data_buf[choice_index].append(content or "")
-                            choice_index += 1
-
-                    # OpenAI responses API
-                    elif hasattr(x, "delta"):
-                        if start_time is not None and ttft is None:
-                            ttft = time.perf_counter() - start_time
-                        if len(data_buf) == 0:
-                            data_buf.append([])
-                        data_buf[0].append(x.delta or "")
-
-                    # OpenAI responses API end of streaming response
-                    if RESPONSES_API_ENABLED and isinstance(x, ResponseCompletedEvent):
-                        _calculate_token_usage(
-                            messages,
-                            x.response,
-                            span,
-                            None,
-                            integration.count_tokens,
-                        )
-                        count_tokens_manually = False
-
-                yield x
-
-            with capture_internal_exceptions():
-                if ttft is not None:
-                    set_data_normalized(
-                        span, SPANDATA.GEN_AI_RESPONSE_TIME_TO_FIRST_TOKEN, ttft
-                    )
-                if len(data_buf) > 0:
-                    all_responses = ["".join(chunk) for chunk in data_buf]
-                    if should_send_default_pii() and integration.include_prompts:
-                        set_data_normalized(
-                            span, SPANDATA.GEN_AI_RESPONSE_TEXT, all_responses
-                        )
-                    if count_tokens_manually:
-                        _calculate_token_usage(
-                            messages,
-                            response,
-                            span,
-                            all_responses,
-                            integration.count_tokens,
-                        )
-            if finish_span:
-                span.__exit__(None, None, None)
-
-        if str(type(response._iterator)) == "<class 'async_generator'>":
-            response._iterator = new_iterator_async()
-        else:
-            response._iterator = new_iterator()
     else:
-        _calculate_token_usage(messages, response, span, None, integration.count_tokens)
+        _calculate_token_usage(input, response, span, None, integration.count_tokens)
         if finish_span:
             span.__exit__(None, None, None)
 
@@ -679,11 +557,315 @@ def _new_chat_completion_common(f: "Any", *args: "Any", **kwargs: "Any") -> "Any
     start_time = time.perf_counter()
     response = yield f, args, kwargs
 
-    _set_output_data(span, response, kwargs, integration, start_time, finish_span=True)
+    is_streaming_response = kwargs.get("stream", False)
+    if is_streaming_response:
+        _set_streaming_completions_api_output_data(
+            span, response, kwargs, integration, start_time, finish_span=True
+        )
+    else:
+        _set_completions_api_output_data(
+            span, response, kwargs, integration, finish_span=True
+        )
 
     return response
 
 
+def _set_completions_api_output_data(
+    span: "Span",
+    response: "Any",
+    kwargs: "dict[str, Any]",
+    integration: "OpenAIIntegration",
+    finish_span: bool = True,
+) -> None:
+    messages = kwargs.get("messages")
+
+    if messages is not None and isinstance(messages, str):
+        messages = [messages]
+
+    _set_common_output_data(
+        span,
+        response,
+        messages,
+        integration,
+        finish_span,
+    )
+
+
+def _set_streaming_completions_api_output_data(
+    span: "Span",
+    response: "Any",
+    kwargs: "dict[str, Any]",
+    integration: "OpenAIIntegration",
+    start_time: "Optional[float]" = None,
+    finish_span: bool = True,
+) -> None:
+    messages = kwargs.get("messages")
+
+    if messages is not None and isinstance(messages, str):
+        messages = [messages]
+
+    ttft: "Optional[float]" = None
+    data_buf: "list[list[str]]" = []  # one for each choice
+
+    old_iterator = response._iterator
+
+    def new_iterator() -> "Iterator[ChatCompletionChunk]":
+        nonlocal ttft
+        for x in old_iterator:
+            span.set_data(SPANDATA.GEN_AI_RESPONSE_MODEL, x.model)
+
+            with capture_internal_exceptions():
+                if hasattr(x, "choices"):
+                    choice_index = 0
+                    for choice in x.choices:
+                        if hasattr(choice, "delta") and hasattr(
+                            choice.delta, "content"
+                        ):
+                            if start_time is not None and ttft is None:
+                                ttft = time.perf_counter() - start_time
+                            content = choice.delta.content
+                            if len(data_buf) <= choice_index:
+                                data_buf.append([])
+                            data_buf[choice_index].append(content or "")
+                        choice_index += 1
+
+            yield x
+
+        with capture_internal_exceptions():
+            if ttft is not None:
+                set_data_normalized(
+                    span, SPANDATA.GEN_AI_RESPONSE_TIME_TO_FIRST_TOKEN, ttft
+                )
+            if len(data_buf) > 0:
+                all_responses = ["".join(chunk) for chunk in data_buf]
+                if should_send_default_pii() and integration.include_prompts:
+                    set_data_normalized(
+                        span, SPANDATA.GEN_AI_RESPONSE_TEXT, all_responses
+                    )
+                _calculate_token_usage(
+                    messages,
+                    response,
+                    span,
+                    all_responses,
+                    integration.count_tokens,
+                )
+
+        if finish_span:
+            span.__exit__(None, None, None)
+
+    async def new_iterator_async() -> "AsyncIterator[ChatCompletionChunk]":
+        nonlocal ttft
+        async for x in old_iterator:
+            span.set_data(SPANDATA.GEN_AI_RESPONSE_MODEL, x.model)
+
+            with capture_internal_exceptions():
+                if hasattr(x, "choices"):
+                    choice_index = 0
+                    for choice in x.choices:
+                        if hasattr(choice, "delta") and hasattr(
+                            choice.delta, "content"
+                        ):
+                            if start_time is not None and ttft is None:
+                                ttft = time.perf_counter() - start_time
+                            content = choice.delta.content
+                            if len(data_buf) <= choice_index:
+                                data_buf.append([])
+                            data_buf[choice_index].append(content or "")
+                        choice_index += 1
+
+            yield x
+
+        with capture_internal_exceptions():
+            if ttft is not None:
+                set_data_normalized(
+                    span, SPANDATA.GEN_AI_RESPONSE_TIME_TO_FIRST_TOKEN, ttft
+                )
+            if len(data_buf) > 0:
+                all_responses = ["".join(chunk) for chunk in data_buf]
+                if should_send_default_pii() and integration.include_prompts:
+                    set_data_normalized(
+                        span, SPANDATA.GEN_AI_RESPONSE_TEXT, all_responses
+                    )
+                _calculate_token_usage(
+                    messages,
+                    response,
+                    span,
+                    all_responses,
+                    integration.count_tokens,
+                )
+
+        if finish_span:
+            span.__exit__(None, None, None)
+
+    if str(type(response._iterator)) == "<class 'async_generator'>":
+        response._iterator = new_iterator_async()
+    else:
+        response._iterator = new_iterator()
+
+
+def _set_responses_api_output_data(
+    span: "Span",
+    response: "Any",
+    kwargs: "dict[str, Any]",
+    integration: "OpenAIIntegration",
+    finish_span: bool = True,
+) -> None:
+    input = kwargs.get("input")
+
+    if input is not None and isinstance(input, str):
+        input = [input]
+
+    _set_common_output_data(
+        span,
+        response,
+        input,
+        integration,
+        finish_span,
+    )
+
+
+def _set_streaming_responses_api_output_data(
+    span: "Span",
+    response: "Any",
+    kwargs: "dict[str, Any]",
+    integration: "OpenAIIntegration",
+    start_time: "Optional[float]" = None,
+    finish_span: bool = True,
+) -> None:
+    input = kwargs.get("input")
+
+    if input is not None and isinstance(input, str):
+        input = [input]
+
+    ttft: "Optional[float]" = None
+    data_buf: "list[list[str]]" = []  # one for each choice
+
+    old_iterator = response._iterator
+
+    def new_iterator() -> "Iterator[ChatCompletionChunk]":
+        nonlocal ttft
+        count_tokens_manually = True
+        for x in old_iterator:
+            with capture_internal_exceptions():
+                if hasattr(x, "delta"):
+                    if start_time is not None and ttft is None:
+                        ttft = time.perf_counter() - start_time
+                    if len(data_buf) == 0:
+                        data_buf.append([])
+                    data_buf[0].append(x.delta or "")
+
+                if isinstance(x, ResponseCompletedEvent):
+                    span.set_data(SPANDATA.GEN_AI_RESPONSE_MODEL, x.response.model)
+
+                    _calculate_token_usage(
+                        input,
+                        x.response,
+                        span,
+                        None,
+                        integration.count_tokens,
+                    )
+                    count_tokens_manually = False
+
+            yield x
+
+        with capture_internal_exceptions():
+            if ttft is not None:
+                set_data_normalized(
+                    span, SPANDATA.GEN_AI_RESPONSE_TIME_TO_FIRST_TOKEN, ttft
+                )
+            if len(data_buf) > 0:
+                all_responses = ["".join(chunk) for chunk in data_buf]
+                if should_send_default_pii() and integration.include_prompts:
+                    set_data_normalized(
+                        span, SPANDATA.GEN_AI_RESPONSE_TEXT, all_responses
+                    )
+                if count_tokens_manually:
+                    _calculate_token_usage(
+                        input,
+                        response,
+                        span,
+                        all_responses,
+                        integration.count_tokens,
+                    )
+
+        if finish_span:
+            span.__exit__(None, None, None)
+
+    async def new_iterator_async() -> "AsyncIterator[ChatCompletionChunk]":
+        nonlocal ttft
+        count_tokens_manually = True
+        async for x in old_iterator:
+            with capture_internal_exceptions():
+                if hasattr(x, "delta"):
+                    if start_time is not None and ttft is None:
+                        ttft = time.perf_counter() - start_time
+                    if len(data_buf) == 0:
+                        data_buf.append([])
+                    data_buf[0].append(x.delta or "")
+
+                if isinstance(x, ResponseCompletedEvent):
+                    span.set_data(SPANDATA.GEN_AI_RESPONSE_MODEL, x.response.model)
+
+                    _calculate_token_usage(
+                        input,
+                        x.response,
+                        span,
+                        None,
+                        integration.count_tokens,
+                    )
+                    count_tokens_manually = False
+
+            yield x
+
+        with capture_internal_exceptions():
+            if ttft is not None:
+                set_data_normalized(
+                    span, SPANDATA.GEN_AI_RESPONSE_TIME_TO_FIRST_TOKEN, ttft
+                )
+            if len(data_buf) > 0:
+                all_responses = ["".join(chunk) for chunk in data_buf]
+                if should_send_default_pii() and integration.include_prompts:
+                    set_data_normalized(
+                        span, SPANDATA.GEN_AI_RESPONSE_TEXT, all_responses
+                    )
+                if count_tokens_manually:
+                    _calculate_token_usage(
+                        input,
+                        response,
+                        span,
+                        all_responses,
+                        integration.count_tokens,
+                    )
+        if finish_span:
+            span.__exit__(None, None, None)
+
+    if str(type(response._iterator)) == "<class 'async_generator'>":
+        response._iterator = new_iterator_async()
+    else:
+        response._iterator = new_iterator()
+
+
+def _set_embeddings_output_data(
+    span: "Span",
+    response: "Any",
+    kwargs: "dict[str, Any]",
+    integration: "OpenAIIntegration",
+    finish_span: bool = True,
+) -> None:
+    input = kwargs.get("input")
+
+    if input is not None and isinstance(input, str):
+        input = [input]
+
+    _set_common_output_data(
+        span,
+        response,
+        input,
+        integration,
+        finish_span,
+    )
+
+
 def _wrap_chat_completion_create(f: "Callable[..., Any]") -> "Callable[..., Any]":
     def _execute_sync(f: "Any", *args: "Any", **kwargs: "Any") -> "Any":
         gen = _new_chat_completion_common(f, *args, **kwargs)
@@ -768,7 +950,9 @@ def _new_embeddings_create_common(f: "Any", *args: "Any", **kwargs: "Any") -> "A
 
         response = yield f, args, kwargs
 
-        _set_output_data(span, response, kwargs, integration, finish_span=False)
+        _set_embeddings_output_data(
+            span, response, kwargs, integration, finish_span=False
+        )
 
         return response
 
@@ -858,7 +1042,15 @@ def _new_responses_create_common(f: "Any", *args: "Any", **kwargs: "Any") -> "An
     start_time = time.perf_counter()
     response = yield f, args, kwargs
 
-    _set_output_data(span, response, kwargs, integration, start_time, finish_span=True)
+    is_streaming_response = kwargs.get("stream", False)
+    if is_streaming_response:
+        _set_streaming_responses_api_output_data(
+            span, response, kwargs, integration, start_time, finish_span=True
+        )
+    else:
+        _set_responses_api_output_data(
+            span, response, kwargs, integration, finish_span=True
+        )
 
     return response
 
