@@ -155,7 +155,8 @@ def get_default_release() -> "Optional[str]":
         return release
 
     for var in (
-        "HEROKU_SLUG_COMMIT",
+        "HEROKU_BUILD_COMMIT",
+        "HEROKU_SLUG_COMMIT",  # deprecated by Heroku, kept for backward compatibility
         "SOURCE_VERSION",
         "CODEBUILD_RESOLVED_SOURCE_VERSION",
         "CIRCLE_SHA1",
